package com.fuyuanshen.app.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.fuyuanshen.common.core.utils.MapstructUtils;
import com.fuyuanshen.common.core.utils.StringUtils;
import com.fuyuanshen.common.mybatis.core.page.TableDataInfo;
import com.fuyuanshen.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import com.fuyuanshen.app.domain.bo.AppRoleBo;
import com.fuyuanshen.app.domain.vo.AppRoleVo;
import com.fuyuanshen.app.domain.AppRole;
import com.fuyuanshen.app.mapper.AppRoleMapper;
import com.fuyuanshen.app.service.IAppRoleService;

import java.util.*;

/**
 * 角色信息Service业务层处理
 *
 * @author Lion Li
 * @date 2025-06-27
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class AppRoleServiceImpl implements IAppRoleService {

    private final AppRoleMapper baseMapper;

    /**
     * 查询角色信息
     *
     * @param roleId 主键
     * @return 角色信息
     */
    @Override
    public AppRoleVo queryById(Long roleId){
        return baseMapper.selectVoById(roleId);
    }

    /**
     * 分页查询角色信息列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 角色信息分页列表
     */
    @Override
    public TableDataInfo<AppRoleVo> queryPageList(AppRoleBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<AppRole> lqw = buildQueryWrapper(bo);
        Page<AppRoleVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的角色信息列表
     *
     * @param bo 查询条件
     * @return 角色信息列表
     */
    @Override
    public List<AppRoleVo> queryList(AppRoleBo bo) {
        LambdaQueryWrapper<AppRole> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<AppRole> buildQueryWrapper(AppRoleBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<AppRole> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(AppRole::getRoleId);
        lqw.like(StringUtils.isNotBlank(bo.getRoleName()), AppRole::getRoleName, bo.getRoleName());
        lqw.eq(StringUtils.isNotBlank(bo.getRoleKey()), AppRole::getRoleKey, bo.getRoleKey());
        lqw.eq(bo.getRoleSort() != null, AppRole::getRoleSort, bo.getRoleSort());
        lqw.eq(StringUtils.isNotBlank(bo.getDataScope()), AppRole::getDataScope, bo.getDataScope());
        lqw.eq(bo.getMenuCheckStrictly() != null, AppRole::getMenuCheckStrictly, bo.getMenuCheckStrictly());
        lqw.eq(bo.getDeptCheckStrictly() != null, AppRole::getDeptCheckStrictly, bo.getDeptCheckStrictly());
        lqw.eq(StringUtils.isNotBlank(bo.getStatus()), AppRole::getStatus, bo.getStatus());
        return lqw;
    }

    /**
     * 新增角色信息
     *
     * @param bo 角色信息
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(AppRoleBo bo) {
        AppRole add = MapstructUtils.convert(bo, AppRole.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setRoleId(add.getRoleId());
        }
        return flag;
    }

    /**
     * 修改角色信息
     *
     * @param bo 角色信息
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(AppRoleBo bo) {
        AppRole update = MapstructUtils.convert(bo, AppRole.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(AppRole entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除角色信息信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }

    @Override
    public Set<String> selectRolePermissionByUserId(Long userId) {
        List<AppRoleVo> perms = baseMapper.selectRolesByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (AppRoleVo perm : perms) {
            if (ObjectUtil.isNotNull(perm)) {
                permsSet.addAll(StringUtils.splitList(perm.getRoleKey().trim()));
            }
        }
        return permsSet;
    }

    /**
     * 根据用户ID查询角色
     *
     * @param userId 用户ID
     * @return 角色列表
     */
    @Override
    public List<AppRoleVo> selectRolesByUserId(Long userId) {
        return baseMapper.selectRolesByUserId(userId);
    }
}
